mod cell;
mod draw;
pub use cell::*;

use crate::fall::FallingTetromino;

pub struct TetrisBoard {
	board: [TetrisCell; TetrisBoard::BOARD_WIDTH * TetrisBoard::BOARD_HEIGHT],
}

impl TetrisBoard {
	const BOARD_WIDTH: usize = 10;
	const BOARD_HEIGHT: usize = 20;

	const CELL_SIZE: usize = 9;

	pub const fn new() -> Self {
		Self {
			board: [TetrisCell::Empty; TetrisBoard::BOARD_WIDTH * TetrisBoard::BOARD_HEIGHT],
		}
	}

	pub fn place_tetromino(&mut self, tetromino: FallingTetromino) {
		for (x, y) in tetromino.tiles() {
			let cell = self.get_cell_mut(x as usize, y as usize);
			if let Some(cell) = cell {
				*cell = TetrisCell::Blue;
			}
		}
	}

	pub fn tetromino_free_below(&self, tetromino: &FallingTetromino) -> bool {
		for (x, y) in tetromino.tiles() {
			let cell = self.get_cell(x as usize, y as usize + 1);
			if cell != Some(&TetrisCell::Empty) {
				return false;
			}
		}

		return true;
	}

	pub fn get_cell(&self, x: usize, y: usize) -> Option<&TetrisCell> {
		if y >= TetrisBoard::BOARD_HEIGHT {
			return None;
		}

		if x >= TetrisBoard::BOARD_WIDTH {
			return None;
		}

		return Some(&self.board[y * TetrisBoard::BOARD_WIDTH + x]);
	}

	pub fn get_cell_mut(&mut self, x: usize, y: usize) -> Option<&mut TetrisCell> {
		if y >= TetrisBoard::BOARD_HEIGHT {
			return None;
		}

		if x >= TetrisBoard::BOARD_WIDTH {
			return None;
		}

		return Some(&mut self.board[y * TetrisBoard::BOARD_WIDTH + x]);
	}
}
